/*
  lp_BLE_TempHumidity
  Reads RH/temp, saves historical values, displays value when connected to by pfodApp, www.pfod.com.au
  (c)2019 Forward Computing and Control Pty. Ltd.
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

/** Supply figures (adjusted for MAX8881 5uA addition to test figures)
    for 2sec advertising interval ~17uA while not connected
    for min/max connections 100mS/150mS and slaveLatency 4 and 30sec display refresh ~25uA
    for each display of the hr or day plot uses ~0.001mAHr
    So if continually connected supply consumption is 0.72mAHr per day (and about 1/3 less if not connected)
    Coin Cell CR2032 235mAHr => 326days ~1year
    Coin Cell CR2450 650mAHr => 900days ~2.5years
    2 x  AAA Alkaline ~1200mAHr to 0.8V say less 20% for 1V cutoff => 1000mAHr => 1389days ~3.8years
    2 x AAA Lithium ~1300mAHr to 1V cutoff => 1805 days ~4.9years
    With Solar assist assume 8hr average per day solar power => 50% increase in battery life

 **/
#include "lp_Si7021.h"
#include <Wire.h>
#include "Button.h"
#include "RHGauge.h"
#include "Thermometer.h"
#include "i2c_ClearBus.h"

char bleNameStr[] = "Office Temp/RH"; // <<<< set BLE advert name here
uint16_t humidityRaw = 0;
uint16_t tempCRaw = 0;
bool sensorIntialized = false; // if not true do not read values
char *errorStr = ""; // sensor initialization error msg, if any.

// store history every 27min for 7days => 380 points
// when connected measure every 30sec to update screen or when screen above buttons is clicked or when the main screen is re-displayed

//#define DEBUG
//#define DUMMY_READINGS

// This sketch needs the Low Power nRF52 installed and compile using Arduino V1.8.2 IDE
// see https://www.forward.com.au/pfod/BLE/LowPower/index.html
#include <lp_BLESerial.h>

// download the pfodParser library V3.10+ from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
// pfodParse.h  includes WStream and then defines Stream as WStream to match the Redbear BLE hardware files and to avoid conflict with mbed::Stream
#include <pfodParser.h>
#include <pfodDwgs.h>

// download the libraries from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
// pfodParser.zip V3.31+ contains pfodParser, pfodSecurity, pfodDelay, pfodBLEBufferedSerial, pfodSMS and pfodRadio

pfodParser parser("V1"); // create a parser to handle the pfod messages
pfodDwgs dwgs(&parser);  // drawing support

const size_t MAX_TO_BE_SENT = 512; // skip next send if already waiting to sent this much
const size_t BLE_BUFFER_SIZE = 1024 + MAX_TO_BE_SENT; // tx buffer size
lp_BLESerial bleSerial(BLE_BUFFER_SIZE); // create a BLE serial connection with a 1024+512 txbuffer
// allow 1024 for responses {..} and 512 for plot raw data sends
// 512 of data takes about 4sec to transmit with these connections settings.
// so pressing the back button while displaying a plot will take less the 5sec to display the main screen
// timeout in pfodApp is 10sec by default for BLE

uint32_t plot_mSOffset = 0; // millis() when {@ response sent

Thermometer thermometer = Thermometer(&dwgs);
RHGauge rhGauge = RHGauge(&dwgs);
Button hrs8PlotButton = Button('m', &dwgs);
Button days1PlotButton =  Button('h', &dwgs);
Button days3PlotButton = Button('d', &dwgs);
Button days7PlotButton =  Button('w', &dwgs);

// there for 1 reading consumes
lp_Si7021 sensor;
lp_timer sensorTimer;
const unsigned long SAMPLE_DELAY_TIME = 60 * 1000; // mS == 1mins
const uint16_t ARRAY_SIZE = 180;//  size 2*4*180*2bytes = 2.8K
// 1min * 180 = 3hrs, 3min = 9hrs, 9min = 27hrs, 27min x 180 = 3.38days,
const uint16_t arraySize_7day = 380; // 27min x 378 = 7.13 days // size 2*1*380*2bytes = 1.5K TOTAL 1.5K+2.8K = 4.3K

// allocate arrays for raw readings uint16_t

struct struct_readings {
  float deltaT_mS; // time step in mins
  int32_t printTime; // for print idx can be -ve for historical readings wraps after 25.85 days
  uint32_t lastSampleTime; // millis() of last sample stored
  uint16_t arraySize; // size of temp and RH arrays
  uint16_t *Temp;
  uint16_t *RH;
  bool printReadingsFinished;// = false; // these are not actually used as first plot is 9hrs at 3min intervals with 1min for first 135 readings
  bool printInitialized;// = false;
  bool clearPlot; // true after re-connect to clear plot and show only new data and historical readings
  uint16_t nextIdx;// = 0; // current index for storing next reading
  uint16_t printCount;// = 0;
  uint16_t printSize; // = 0;
  uint16_t printIdx;// = 0;
  uint16_t startIdx;// = 0;
  const char* dataPadding; // string to add to plot data to distingush the 4 plots
  const char* name; // name of this readings
};

struct struct_readings readings_1min;
const char dataPadding_1min[] = ",1m"; // nothing not used
const char name_1min[] = "1min";
uint16_t Temp_1min[ARRAY_SIZE];
uint16_t RH_1min[ARRAY_SIZE];

struct struct_readings readings_3min;
const char dataPadding_3min[] = ",,3m";
const char name_3min[] = "3min";
uint16_t Temp_3min[ARRAY_SIZE];
uint16_t RH_3min[ARRAY_SIZE];

struct struct_readings readings_9min;
const char dataPadding_9min[] = ",,,9m";
const char name_9min[] = "9min";
uint16_t Temp_9min[ARRAY_SIZE];
uint16_t RH_9min[ARRAY_SIZE];

struct struct_readings readings_27min;
const char dataPadding_27min[] = ",,,,27m";
const char name_27min[] = "27min";

uint16_t Temp_27min[arraySize_7day];
uint16_t RH_27min[arraySize_7day];

struct struct_readings readings_27minExt;  // this actually just a longer array of 27mins
const char dataPadding_7day[] = ",,,,,27ext";
const char name_7day[] = "27ext";


const uint16_t SIZE_OF_STRUCTS_ARRAY = 5; // 1,3,9,27, 7day (handled at end)
struct struct_readings* arrayOfStructReadings[SIZE_OF_STRUCTS_ARRAY];
struct_readings* currentReadingsPrint; // stores indices


lp_timer readResultDelay;
const unsigned long READ_RESULT_DELAY_TIME = 25; // mS

lp_timer PrintDelay;
const unsigned long PRINT_DELAY_TIME = 50; // mS

bool sendReadingsPaused = false;

void incrementNextIdx(struct_readings* readingsPtr) {
  uint16_t idx = readingsPtr->nextIdx;
  idx++;
  if (idx >= readingsPtr->arraySize) {
    idx = 0; // loop
  }
  readingsPtr->nextIdx = idx;
}

uint16_t decrementIdx(struct_readings* readingsPtr, uint16_t idx) {
  if (idx == 0) {
    idx = readingsPtr->arraySize; // loop
  }
  idx--;
  return idx;
}


void updateReadings(uint16_t tempCRaw, uint16_t humidityRaw) {
  uint32_t mS = millis();
#ifdef DUMMY_READINGS
  if (tempCRaw == 0) {
    tempCRaw += 10; // make sure any sensor reading is not zero
  }
#endif

  parser.print(mS - plot_mSOffset); // this sample's mS
  parser.print(','); parser.print(sensor.Temp_RawToFloat(tempCRaw));
  parser.print(','); parser.print(sensor.RH_RawToInt(humidityRaw));
  parser.print(readings_1min.dataPadding); // get padding for next plot and send the finer info to it
  parser.println(); // end of CSV data record

  struct_readings* readingsPtr;
  uint16_t arrayIdx = 0;
  uint16_t currentIdx; // index of last stored value
  // only loop over 1,3,9,27 skip 81 handled below
  for (; arrayIdx < (SIZE_OF_STRUCTS_ARRAY - 1); arrayIdx++) {
    readingsPtr = arrayOfStructReadings[arrayIdx];
    currentIdx = readingsPtr->nextIdx;
    readingsPtr->Temp[currentIdx] = tempCRaw;
    readingsPtr->RH[currentIdx] = humidityRaw;
    readingsPtr->lastSampleTime = mS;
    // if get new reading while sending history, update startHrIdx to prevent new reading being shown at end of history
    readingsPtr->startIdx = currentIdx;
    // send this new reading
    // note lp_BLESerial drops data if not connected so can always just send here
    // get padding for next plot and send the finer info to it
    struct_readings* readingsPtrNext = arrayOfStructReadings[arrayIdx + 1];
    parser.print(mS - plot_mSOffset); // this sample's mS
    parser.print(','); parser.print(sensor.Temp_RawToFloat(tempCRaw));
    parser.print(','); parser.print(sensor.RH_RawToInt(humidityRaw));
    parser.print(readingsPtrNext->dataPadding); // get padding for next plot and send the finer info to it
    parser.println(); // end of CSV data record

    incrementNextIdx(readingsPtr); // move on to next idx
    if ((currentIdx % 3) != 0) { // is last stored was multiple of 3 loop to add to next set of readings
      if (readingsPtr == &readings_27min) {
        // send 27ext data also as we just sent 27min fine reading
        parser.print(mS - plot_mSOffset); // this sample's mS
        parser.print(','); parser.print(sensor.Temp_RawToFloat(tempCRaw));
        parser.print(','); parser.print(sensor.RH_RawToInt(humidityRaw));
        parser.print(readings_27minExt.dataPadding); // get padding for next plot and send the finer info to it
        parser.println(); // end of CSV data record
      }
      break; // finished
    }
    // else loop and add to next set of readings
  }
  // always keep these in sync
  if (readingsPtr == &readings_27min) {
    // just updated readings_27min
    readings_27minExt.nextIdx = readings_27min.nextIdx;
    readings_27minExt.startIdx = readings_27min.startIdx;
    readings_27minExt.lastSampleTime = readings_27min.lastSampleTime;
  }
}

void initializeArray(struct_readings* readings) {
  for (uint16_t i = 0; i < ARRAY_SIZE; i++) {
    readings->Temp[i] = readings->RH[i] = 0; // 0 is invalid
  }
  //readings->printFactor = 1.0f; //default
  readings->nextIdx = 0; // current index for storing next reading
  readings->printReadingsFinished = false;
  readings->printInitialized = false;
}

#ifdef DUMMY_READINGS
uint16_t base = 1000;
void dummyReadings() {
  int noReadings = 10260 + 3; //380 x 27min in 1min readings
  for (int i = 0; i < noReadings; i++) {
    updateReadings(base, base);
    base += 10;
  }
#ifdef DEBUG
  Serial.println();
  Serial.println("Finished loading dummy readings");
#endif
}
#endif

void setup() {
#ifdef DEBUG
  Serial.begin(115200);
  Serial.println();
  for (int i = 10; i > 0; i--) {
    Serial.print(' '); Serial.print(i);
    delay(500);
  }
  Serial.println();
#endif


#ifdef DEBUG
  Serial.println();
  Serial.println("I2C_ClearBus (waits 2.5sec)");
#endif

  int rtn = I2C_ClearBus(); // clear the I2C bus first before calling Wire.begin()
  if (rtn != 0) {
#ifdef DEBUG
    Serial.println(F("I2C bus error. Could not clear"));
#endif
    if (rtn == 1) {
      errorStr = "I2C bus error\nSCL clock line held low";
    } else if (rtn == 2) {
      errorStr = "I2C bus error\nSCL clock line held low by slave clock stretch";
    } else if (rtn == 3) {
      errorStr = "I2C bus error\nSDA data line held low";
    }
  } else {
    //Initialize the sensor and ping it
    if (sensor.begin()) {
#ifdef DEBUG
      Serial.println("Found Si7021");
#endif
      sensorIntialized = true; // else leave as false
    } else {
      errorStr = "Error\n Si7021 not found";
    }
  }
#ifdef DEBUG
  Serial.println(errorStr);
#endif

  rhGauge.setLabel(F("RH\n"));
  rhGauge.setValue(0);
  thermometer.setValue(0); // 150 == 27.5. -30 = -17.5, 300 = 60.5
  hrs8PlotButton.setLabel(F("Last 8 hrs"));
  days1PlotButton.setLabel(F("Last 24 hrs"));
  days3PlotButton.setLabel(F("Last 3 days"));
  days7PlotButton.setLabel(F("Last 7 days"));


  initializeArray(&readings_1min);
  readings_1min.dataPadding = dataPadding_1min;
  readings_1min.name = name_1min;
  readings_1min.deltaT_mS = SAMPLE_DELAY_TIME;
  readings_1min.Temp = Temp_1min;
  readings_1min.RH = RH_1min;
  readings_1min.arraySize = ARRAY_SIZE;
  readings_1min.printSize = ARRAY_SIZE;

  initializeArray(&readings_3min);
  readings_3min.dataPadding = dataPadding_3min;
  readings_3min.name = name_3min;
  readings_3min.deltaT_mS = 3 * readings_1min.deltaT_mS;
  readings_3min.Temp = Temp_3min;
  readings_3min.RH = RH_3min;
  readings_3min.arraySize = ARRAY_SIZE;
  readings_3min.printSize = ARRAY_SIZE;

  initializeArray(&readings_9min);
  readings_9min.dataPadding = dataPadding_9min;
  readings_9min.name = name_9min;
  readings_9min.deltaT_mS = 3 * readings_3min.deltaT_mS;
  readings_9min.Temp = Temp_9min;
  readings_9min.RH = RH_9min;
  readings_9min.arraySize = ARRAY_SIZE;
  readings_9min.printSize = ARRAY_SIZE;

  initializeArray(&readings_27min);
  readings_27min.dataPadding = dataPadding_27min;
  readings_27min.name = name_27min;
  readings_27min.deltaT_mS = 3 * readings_9min.deltaT_mS;
  readings_27min.Temp = Temp_27min;
  readings_27min.RH = RH_27min;
  readings_27min.arraySize = arraySize_7day;
  readings_27min.printSize = ARRAY_SIZE; // limit this print to 180

  initializeArray(&readings_27minExt);
  readings_27minExt.dataPadding = dataPadding_7day;
  readings_27minExt.name = name_7day;
  readings_27minExt.deltaT_mS = readings_27min.deltaT_mS;
  readings_27minExt.Temp = Temp_27min;
  readings_27minExt.RH = RH_27min;
  readings_27minExt.arraySize = arraySize_7day;
  readings_27minExt.printSize = arraySize_7day;

  arrayOfStructReadings[0] = &readings_1min;
  arrayOfStructReadings[1] = &readings_3min;
  arrayOfStructReadings[2] = &readings_9min;
  arrayOfStructReadings[3] = &readings_27min;
  arrayOfStructReadings[4] = &readings_27minExt; // used for padding for sends


  // set advertised name
  bleSerial.setName(bleNameStr);  // <<<<<<<< set your device name here
  // begin BLE initialization
  // BLE advertising and connection defaults are in bleConstants.h

  bleSerial.setAdvertisingInterval(1000); // in mS, default is 500, 1000 gives ~34uA while advertising, 2000 gives ~22uA  while advertising
  // 500 to 1000 is the suggested value for advertising interval

  // connection Interval
  // default is 100mS min 150mS max
  // bleSerial.setConnectionInterval(100,150); //leave as default, this suggests a connection interval to be used by the Android mobile,
  // the Android mobile actually chooses the connection interval to be used
  // while downloading 36hrs plot have ~110uA for 45sec => extra 85uA for 45sec == ~0.001mAHr per plot display

  // set the slave latency default is 0, i.e. respond to every connection msg
  bleSerial.setSlaveLatency(4); //default is 0, 0=>~50uA, 2=>~30uA, 4 => ~25uA + refresh every 30sec 66uA for 2sec => ~4.5uA each 30sec refresh for total of ~30uA connected

  bleSerial.begin();

  // you can set TX power here AFTER calling bleSerial.begin()
  // default is +4
  // bleSerial.setTxPower(4);

  parser.connect(&bleSerial);

  if (sensorIntialized) {
    sensorTimer.startTimer(SAMPLE_DELAY_TIME, startRHandTempConversion);
  } else {
#ifdef DUMMY_READINGS
    sensorTimer.startTimer(SAMPLE_DELAY_TIME, dummyRHandTempConversion);  // start dummy readings
#endif
  }
  // do an initial sensor reading to initialize WIRE interface, otherwise will draw 218uA until first reading
  startRHandTempConversion();

#ifdef DUMMY_READINGS
  dummyReadings();
#endif

#ifdef DEBUG
  Serial.println("setup() finished");
#endif
}

bool dwgRefresh = false;


void printCurrentReadings() {
  if (!currentReadingsPrint->printInitialized) { // initialize if need to
    currentReadingsPrint->printInitialized = true;
#ifdef DEBUG
    Serial.print("printCurrentReadings "); Serial.print(" Print:"); Serial.print(currentReadingsPrint->name);   Serial.println();
    Serial.print(currentReadingsPrint->name); Serial.print(" startIdx:");  Serial.print(currentReadingsPrint->startIdx);  Serial.print(" printSize:"); Serial.print(currentReadingsPrint->printSize); Serial.println();
#endif
    // start at previous nextIdx and go back in time
    currentReadingsPrint->printCount = 0;
    currentReadingsPrint->printTime = currentReadingsPrint->lastSampleTime - plot_mSOffset; // starting time is delta mS since {@}
    currentReadingsPrint->printIdx = decrementIdx(currentReadingsPrint, currentReadingsPrint->nextIdx); // most current reading
    currentReadingsPrint->startIdx = currentReadingsPrint->printIdx; // stop when we get here again
  }
  sendReadingsPaused = false;
  printNextReading();
}

void printNextReading() {
#ifdef DEBUG
  Serial.print("printNextReading For  "); Serial.print(currentReadingsPrint->name); Serial.print(" toBeSent:"); Serial.println(bleSerial.bytesToBeSent()); Serial.println();
#endif
  if (currentReadingsPrint->printReadingsFinished) {
    return;
  }
  if (sendReadingsPaused) {
    return;
  }

  if (bleSerial.bytesToBeSent() > MAX_TO_BE_SENT) {
    // skip this send and wait a bit longer
    PrintDelay.startDelay(2 * PRINT_DELAY_TIME, printNextReading);
#ifdef DEBUG
    Serial.print("skip");  Serial.println();
#endif
    return;
  }


  bool sentReading = false;

#ifdef DEBUG
  Serial.print("printing "); Serial.print(currentReadingsPrint->name); Serial.println();
#endif
  if (currentReadingsPrint->Temp[currentReadingsPrint->printIdx] != 0) { // skip invalid reading in first hr
    parser.print(currentReadingsPrint->printTime); // time in mS +/-
    parser.print(','); parser.print(sensor.Temp_RawToFloat(currentReadingsPrint->Temp[currentReadingsPrint->printIdx]));
    parser.print(','); parser.print(sensor.RH_RawToInt(currentReadingsPrint->RH[currentReadingsPrint->printIdx]));
    parser.print(currentReadingsPrint->dataPadding); // same padding for fine
    parser.println(); // end of CSV data record
    sentReading = true;
#ifdef DEBUG
    Serial.print(" sendReading:"); Serial.print(sentReading);
    Serial.print(" printIdx:"); Serial.print(currentReadingsPrint->printIdx);
    Serial.print(" Time:"); Serial.print(currentReadingsPrint->printTime);
    Serial.print(" Temp:"); Serial.print(currentReadingsPrint->Temp[currentReadingsPrint->printIdx]);
    Serial.println();
#endif

    currentReadingsPrint->printIdx = decrementIdx(currentReadingsPrint, currentReadingsPrint->printIdx);
    currentReadingsPrint->printCount++;
    currentReadingsPrint->printTime -= currentReadingsPrint->deltaT_mS;
  }
#ifdef DEBUG
  Serial.print("sentReading "); Serial.print(sentReading); Serial.print(" "); Serial.print(currentReadingsPrint->printIdx); Serial.print(" != "); Serial.print(currentReadingsPrint->startIdx); Serial.println();
#endif
  if (sentReading && (currentReadingsPrint->printCount < currentReadingsPrint->printSize) && (currentReadingsPrint->printIdx != currentReadingsPrint->startIdx) ) {
#ifdef DEBUG
    Serial.print("startDelay "); Serial.println();
#endif
    PrintDelay.startDelay(PRINT_DELAY_TIME, printNextReading);
  } else {
    // finished
#ifdef DEBUG
    Serial.print("print finished "); Serial.println();
#endif
    currentReadingsPrint->printReadingsFinished = true;
  }

}

void sendClearPlotIfNecessary(struct_readings * readings) {
  if (readings->clearPlot) {
    readings->clearPlot = false;
    parser.print(F("~C"));
  }
}

void loop() {
  sleep(); // just sleep here waiting for the timer to trigger
  // check ble serial when triggered by anything
  uint8_t cmd = parser.parse(); // parse incoming data from connection
  // parser returns non-zero when a pfod command is fully parsed
  if (cmd != 0) { // have parsed a complete msg { to }
    uint8_t* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
    pfod_MAYBE_UNUSED(pfodFirstArg); // may not be used, just suppress warning
    long pfodLongRtn; // used for parsing long return arguments, if any
    pfod_MAYBE_UNUSED(pfodLongRtn); // may not be used, just suppress warning
    if ('.' == cmd) {
      // pfodApp has connected and sent {.} , it is asking for the main menu
      pauseSendingReadings();
      if (!parser.isRefresh()) {
        sendMainMenu(); // send back the menu designed
      } else {
        sendMainMenuUpdate(); // menu is cached just send update
      }
      // handle {@} request
    } else if ('@' == cmd) { // pfodApp requested 'current' time
      clearPlots(); // next plot starts with historical data only
      plot_mSOffset = millis(); // capture current millis as offset rawdata timestamps
      parser.print(F("{@`0}")); // return `0 as 'current' raw data milliseconds
      stopSendingReadings();


    } else if ('A' == cmd) {
      // in the main Menu of Menu_1
      byte dwgCmd = parser.parseDwgCmd(); // parse rest of dwgCmd, return first char of active cmd
      if (dwgCmd == hrs8PlotButton.getCmd()) { // 8hrs plot at 3min intervals + 1mins
        currentReadingsPrint = &readings_3min;
        parser.print(F("{=8hrs "));
        parser.print(F("Temperature / RH~S~E HH:mm:ss"));
        sendClearPlotIfNecessary(currentReadingsPrint);
        parser.print(F("|day/time|Temp C~32~8~deg C|RH %~87~37~%"));
        parser.print(F("||}"));
        printCurrentReadings(); // start printing readings

      } else if (dwgCmd == days1PlotButton.getCmd()) { // 24hrs plot at 9min + 3mins
        currentReadingsPrint = &readings_9min;
        parser.print(F("{=24hrs "));
        parser.print(F("Temperature / RH~S~E HH:mm:ss"));
        sendClearPlotIfNecessary(currentReadingsPrint);
        parser.print(F("|day/time|Temp C~32~8~deg C|RH %~87~37~%"));
        parser.print(F("|||}"));
        printCurrentReadings(); // start printing readings

      } else if (dwgCmd == days3PlotButton.getCmd()) { // 3day plot at 27min + 9mins
        currentReadingsPrint = &readings_27min;
        parser.print(F("{=3 days "));
        parser.print(F("Temperature / RH~S~E HH:mm:ss"));
        sendClearPlotIfNecessary(currentReadingsPrint);
        parser.print(F("|day/time|Temp C~32~8~deg C|RH %~87~37~%"));
        parser.print(F("||||}"));
        printCurrentReadings(); // start printing readings use smaller print size for this 3 day plot

      } else if (dwgCmd == days7PlotButton.getCmd()) { // 10day plot at 81min + 27mins
        currentReadingsPrint = &readings_27minExt;
        parser.print(F("{=7 days "));
        parser.print(F("Temperature / RH~S~E MMM/dd HH:mm"));
        sendClearPlotIfNecessary(currentReadingsPrint);
        parser.print(F("|date/time|Temp C~32~8~deg C|RH %~87~37~%"));
        parser.print(F("|||||}"));
        printCurrentReadings(); // start printing readings

      } else {
        dwgRefresh = true; // just send update
        startRHandTempDwgUpdateConversion(); // will send dwg update when conversion complete in 25mS
        //sendDrawingUpdates_z(); // update the drawing
      }
      // always send back a response or pfodApp will timeout

    } else if ('z' == cmd) {
      pauseSendingReadings();
      // get starting offset
      dwgRefresh = parser.isRefresh(); // cleared when update sent
      startRHandTempDwgUpdateConversion(); // will send dwg update when conversion complete in 25mS

    } else if ('!' == cmd) {
      pauseSendingReadings();
      // CloseConnection command
      closeConnection(parser.getPfodAppStream());
    } else {
      // keep alive processed here so do not pauseSendingReadings();
      // or unknown command
      parser.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
    }
  }
  if (!bleSerial.isConnected()) {
    stopSendingReadings(); // reset -- next connection will send fill history
  }
}

void pauseSendingReadings() {
  sendReadingsPaused = true;
}

void stopSending(struct_readings * readings) {
  readings->printReadingsFinished = false; // stop sending readings
  readings->printInitialized = false; // re-initialize next connection
}

void stopSendingReadings() {
  // stops sending all readings
  stopSending(&readings_1min);
  stopSending(&readings_3min);
  stopSending(&readings_9min);
  stopSending(&readings_27min);
  stopSending(&readings_27minExt);
}

void clearPlots() {
  // stops sending all readings
  readings_1min.clearPlot = true;
  readings_3min.clearPlot = true;
  readings_9min.clearPlot = true;
  readings_27min.clearPlot = true;
  readings_27minExt.clearPlot = true;
}


void closeConnection(Stream * io) {
  // add any special code here to force connection to be dropped
  stopSendingReadings();
  ((lp_BLESerial*)io)->close();
}

#ifdef DUMMY_READINGS
void dummyRHandTempConversion() {
  updateReadings(base, base);
  base += 10;
}
#endif

bool conversionRunning = false;
void startRHandTempConversion() {
  if (!conversionRunning) {
    conversionRunning = true;
    if (sensorIntialized) {
      sensor.startRHandTemp();
    }
  } // else just schedule to pickup last result
  readResultDelay.startDelay(READ_RESULT_DELAY_TIME, readRHResults);
}

void startRHandTempDwgUpdateConversion() {
  if (!conversionRunning) {
    conversionRunning = true;
    if (sensorIntialized) {
      sensor.startRHandTemp();
    }
  } // else just schedule to pickup last result
  readResultDelay.startDelay(READ_RESULT_DELAY_TIME, readRHResultsAndUpdateDwg);
}



uint16_t currentTemp_Raw;
uint16_t currentRH_Raw;

void readRHResultsAndUpdateDwg() {
  if (sensorIntialized) {
    // Get Result of Relative Humidity from or Si7021
    currentRH_Raw = sensor.readRH_Raw();
    // Read Result of Temperature from RH measurement Si7021
    currentTemp_Raw = sensor.readLastRHTemp_Raw();
  }
  conversionRunning = false;
  float temp = sensor.Temp_RawToFloat(currentTemp_Raw);
  int rh = sensor.RH_RawToInt(currentRH_Raw);
  // convert rh to in in range 0 to rhGauge.RANGE
  int rhValue = (int)(rh * (rhGauge.RANGE / 100.0f));
  rhGauge.setValue(rhValue);

  // convert temp into range 0 to Thermometer.RANGE
  int tempValue = floor(( (temp - thermometer.MIN_C) * (thermometer.RANGE / (thermometer.MAX_C - thermometer.MIN_C)) ) + 0.5);
  thermometer.setValue(tempValue);

#ifdef DEBUG
  Serial.print(" RH%: "); Serial.print(rh);
  Serial.print(" rhValue: "); Serial.print(rhValue);
  Serial.print(" degC: "); Serial.print(temp);
  Serial.print(" tempValue: "); Serial.print(tempValue);
  Serial.println();
  Serial.print("Temp:");  Serial.print(sensor.Temp_RawToFloat(currentTemp_Raw));  Serial.print("C, ");
  Serial.print("Humidity:");  Serial.print(sensor.RH_RawToInt(currentRH_Raw));  Serial.print("%");
  Serial.println();
#endif

  if (!dwgRefresh) {
    sendDrawing_z();
  } else {
    sendDrawingUpdates_z();
  }
  dwgRefresh = false;
}

void readRHResults() {
  if (sensorIntialized) {
    // Get Result of Relative Humidity from or Si7021
    humidityRaw = sensor.readRH_Raw();
    // Read Result of Temperature from RH measurement Si7021
    tempCRaw = sensor.readLastRHTemp_Raw();
  }
  conversionRunning = false;
  // Temperature is measured every time RH is requested.
  // It is faster, therefore, to read it from previous RH
  // measurement with readLastRHTemp() instead with startTemp() / readTemp()
  // save 1min reading and every 3 readings save in next array
  // i.e. save and increment and if new idx % 3 == 0 save in next array etc
#ifndef DUMMY_READINGS
  updateReadings(tempCRaw, humidityRaw);
#endif
}


void sendMainMenu() {
  // !! Remember to change the parser version string
  //    every time you edit this method
  parser.print(F("{,"));  // start a Menu screen pfod message
  // send menu background, format, prompt, refresh and version
  if (sensorIntialized) {
    parser.print(F("<bg w>~<+2><r><b>Temperature</r>\n <-2>and</-2> <bl><b>Rel Humidity")); // keep alive 5sec by default
  } else {
    parser.print(F("<bg w>~<+2><r><b>"));
    parser.print(errorStr);
  }
  parser.sendRefreshAndVersion(0); //  no refresh, dwg refreshes itself
  // send menu items
  parser.print(F("|+A~z")); //the dwg
  parser.print(F("}"));  // close pfod message
}

void sendMainMenuUpdate() {
  parser.print(F("{;"));  // start an Update Menu pfod message
  if (sensorIntialized) {
  } else {
    parser.print(F("<bg w>~<+2><r><b>Error: Si7021\ndid not Initialize"));
  }
  parser.print(F("|A")); //the dwg
  parser.print(F("}"));  // close pfod message
  // ============ end of menu ===========
}

void sendDrawing_z() {
  dwgs.start(50, 60, dwgs.WHITE); // background defaults to WHITE if omitted i.e.  start(50,60);
  parser.sendRefreshAndVersion(30000); // re-request dwg every 30sec sec. this is ignored if no parser version set
  // touch above buttons to force display updates
  dwgs.touchZone().cmd('u').size(50, 39).send();
  dwgs.pushZero(35, 22, 1.5); // move zero to centre of dwg to 35,22 and scale by 1.5 times
  rhGauge.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(18, 33); // move zero to centre of dwg to 18,33 scale is 1 (default)
  thermometer.draw(); // draw the control
  dwgs.popZero();

  dwgs.pushZero(12.5, 43, 0.7); // move zero to centre of dwg to 12.5,43 and scale by 0.7
  hrs8PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(37.5, 43, 0.7); // move zero to centre of dwg to 37.5,43 and scale by 0.7
  days1PlotButton.draw(); // draw the control
  dwgs.popZero();

  dwgs.pushZero(12.5, 54, 0.7); // move zero to centre of dwg to 12.5,54 and scale by 0.7
  days3PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.pushZero(37.5, 54, 0.7); // move zero to centre of dwg to 37.5,54 and scale by 0.7
  days7PlotButton.draw(); // draw the control
  dwgs.popZero();
  dwgs.end();
}

void sendDrawingUpdates_z() {
  dwgs.startUpdate();
  rhGauge.update(); // update with current state
  thermometer.update(); // update with current state
  dwgs.end();
}
